﻿//*****************************************************************************
//
//  File:       GalleryPropertiesProvider.cs
//
//  Contents:   Definition for gallery properties provider 
//
//*****************************************************************************

using BitFlex.Windows.Forms.OfficeRibbon.Interop;

namespace BitFlex.Windows.Forms.OfficeRibbon.Controls.Properties
{
    /// <summary>
    /// Definition for gallery properties provider interface
    /// </summary>
    public interface IGalleryPropertiesProvider
    {
        /// <summary>
        /// Categories property
        /// </summary>
        IUICollection Categories { get; }

        /// <summary>
        /// Items source property
        /// </summary>
        IUICollection ItemsSource { get; }

        /// <summary>
        /// Selected item property
        /// </summary>
        uint SelectedItem { get; set; }
    }

    /// <summary>
    /// Implementation of IGalleryPropertiesProvider
    /// </summary>
    public class GalleryPropertiesProvider : BasePropertiesProvider, IGalleryPropertiesProvider
    {
        /// <summary>
        /// GalleryPropertiesProvider ctor
        /// </summary>
        /// <param name="ribbon">parent ribbon</param>
        /// <param name="commandId">ribbon control command id</param>
        public GalleryPropertiesProvider(Ribbon ribbon, uint commandId)
            : base(ribbon, commandId)
        { 
            // add supported properties
            _supportedProperties.Add(RibbonProperties.Categories);
            _supportedProperties.Add(RibbonProperties.ItemsSource);
            _supportedProperties.Add(RibbonProperties.SelectedItem);
        }

        private IUICollection _categories = new UICollection();
        private IUICollection _itemsSource = new UICollection();
        private uint? _selectedItem;

        /// <summary>
        /// Handles IUICommandHandler.UpdateProperty function for the supported properties
        /// </summary>
        /// <param name="key">The Property Key to update</param>
        /// <param name="currentValue">A pointer to the current value for key. This parameter can be NULL</param>
        /// <param name="newValue">When this method returns, contains a pointer to the new value for key</param>
        /// <returns>Returns S_OK if successful, or an error value otherwise</returns>
        public override HRESULT UpdateProperty(ref PropertyKey key, PropVariantRef currentValue, ref PropVariant newValue)
        {
            if (key == RibbonProperties.Categories)
            {
                if (_categories != null)
                {
                    IUICollection categories = (IUICollection)currentValue.PropVariant.Value;

                    categories.Clear();
                    uint count;
                    _categories.GetCount(out count);
                    for (uint i = 0; i < count; ++i)
                    { 
                        object item;
                        _categories.GetItem(i, out item);
                        categories.Add(item);
                    }
                }
            }
            else if (key == RibbonProperties.ItemsSource)
            {
                if (_itemsSource != null)
                {
                    IUICollection itemsSource = (IUICollection)currentValue.PropVariant.Value;

                    itemsSource.Clear();
                    uint count;
                    _itemsSource.GetCount(out count);
                    for (uint i = 0; i < count; ++i)
                    {
                        object item;
                        _itemsSource.GetItem(i, out item);
                        itemsSource.Add(item);
                    }
                }
            }
            else if (key == RibbonProperties.SelectedItem)
            {
                if (_selectedItem.HasValue)
                {
                    newValue.SetUInt(_selectedItem.Value);
                }
            }

            return HRESULT.S_OK;
        }

        #region IGalleryPropertiesProvider Members

        /// <summary>
        /// Categories property
        /// </summary>
        public IUICollection Categories
        {
            get
            {
                if (_ribbon.Initalized)
                {
                    PropVariant unknownValue;
                    HRESULT hr = _ribbon.Framework.GetUICommandProperty(_commandID, ref RibbonProperties.Categories, out unknownValue);
                    if (NativeMethods.Succeeded(hr))
                    {
                        return (IUICollection)unknownValue.Value;
                    }
                }

                return _categories;
            }
        }

        /// <summary>
        /// Items source property
        /// </summary>
        public IUICollection ItemsSource
        {
            get
            {
                if (_ribbon.Initalized)
                {
                    PropVariant unknownValue;
                    HRESULT hr = _ribbon.Framework.GetUICommandProperty(_commandID, ref RibbonProperties.ItemsSource, out unknownValue);
                    if (NativeMethods.Succeeded(hr))
                    {
                        return (IUICollection)unknownValue.Value;
                    }
                }

                return _itemsSource;
            }
        }

        /// <summary>
        /// Selected item property
        /// </summary>
        public uint SelectedItem
        {
            get
            {
                if (_ribbon.Initalized)
                {
                    PropVariant uintValue;
                    HRESULT hr = _ribbon.Framework.GetUICommandProperty(_commandID, ref RibbonProperties.SelectedItem, out uintValue);
                    if (NativeMethods.Succeeded(hr))
                    {
                        return (uint)uintValue.Value;
                    }
                }

                return _selectedItem.GetValueOrDefault(Constants.UI_Collection_InvalidIndex);
            }
            set
            {
                _selectedItem = value;

                if (_ribbon.Initalized)
                {
                    PropVariant uintValue = PropVariant.FromObject(value);
                    HRESULT hr = _ribbon.Framework.SetUICommandProperty(_commandID, ref RibbonProperties.SelectedItem, ref uintValue);
                }
            }
        }
    
        #endregion
    }
}
